@@ -23,4 +23,4 @@ UNICAST = 1
 MULTICAST = 2
 MPLS_LABEL = 4  # RFC 3107
 MPLS_VPN = 128  # RFC 4364
-ROUTE_TARGET_CONSTRTAINS = 132  # RFC 4684
+ROUTE_TARGET_CONSTRAINTS = 132  # RFC 4684
