@@ -21,10 +21,13 @@ OFP_HEADER_PACK_STR = '!BBHI'
 OFP_HEADER_SIZE = 8
 assert calcsize(OFP_HEADER_PACK_STR) == OFP_HEADER_SIZE
 
-# note: while IANA assigned port number for OpenFlow is 6653,
-# 6633 is (still) the defacto standard.
-OFP_TCP_PORT = 6633
-OFP_SSL_PORT = 6633
+# Note: IANA assigned port number for OpenFlow is 6653
+# from OpenFlow 1.3.3 (EXT-133).
+# Some applications may still use 6633 as the de facto standard though.
+OFP_TCP_PORT = 6653
+OFP_SSL_PORT = 6653
+OFP_TCP_PORT_OLD = 6633
+OFP_SSL_PORT_OLD = 6633
 
 # Vendor/Experimenter IDs
 # https://rs.opennetworking.org/wiki/display/PUBLIC/ONF+Registry
