@@ -34,8 +34,6 @@ from ryu.services.protocols.bgp.rtconf.common \
     import DEFAULT_REFRESH_MAX_EOR_TIME
 from ryu.services.protocols.bgp.rtconf.common \
     import DEFAULT_REFRESH_STALEPATH_TIME
-from ryu.services.protocols.bgp.rtconf.common \
-    import DEFAULT_BGP_CONN_RETRY_TIME
 from ryu.services.protocols.bgp.rtconf.common import DEFAULT_LABEL_RANGE
 from ryu.services.protocols.bgp.rtconf.common import REFRESH_MAX_EOR_TIME
 from ryu.services.protocols.bgp.rtconf.common import REFRESH_STALEPATH_TIME
@@ -57,8 +55,6 @@ from ryu.services.protocols.bgp.rtconf.neighbors \
 from ryu.services.protocols.bgp.rtconf.neighbors import DEFAULT_CONNECT_MODE
 from ryu.services.protocols.bgp.rtconf.neighbors import PEER_NEXT_HOP
 from ryu.services.protocols.bgp.rtconf.neighbors import PASSWORD
-from ryu.services.protocols.bgp.rtconf.neighbors import IN_FILTER
-from ryu.services.protocols.bgp.rtconf.neighbors import OUT_FILTER
 from ryu.services.protocols.bgp.rtconf.neighbors import IS_ROUTE_SERVER_CLIENT
 from ryu.services.protocols.bgp.rtconf.neighbors import IS_NEXT_HOP_SELF
 from ryu.services.protocols.bgp.rtconf.neighbors import CONNECT_MODE
@@ -244,7 +240,8 @@ class BGPSpeaker(object):
                      next_hop=None, password=None, multi_exit_disc=None,
                      site_of_origins=None, is_route_server_client=False,
                      is_next_hop_self=False, local_address=None,
-                     local_port=None, connect_mode=DEFAULT_CONNECT_MODE):
+                     local_port=None, local_as=None,
+                     connect_mode=DEFAULT_CONNECT_MODE):
         """ This method registers a new neighbor. The BGP speaker tries to
         establish a bgp session with the peer (accepts a connection
         from the peer and also tries to connect to it).
@@ -265,11 +262,14 @@ class BGPSpeaker(object):
         ``enable_vpnv6`` enables VPNv6 address family for this
         neighbor. The default is False.
 
+        ``enable_enhanced_refresh`` enable Enhanced Route Refresh for this
+        neighbor. The default is False.
+
         ``next_hop`` specifies the next hop IP address. If not
         specified, host's ip address to access to a peer is used.
 
         ``password`` is used for the MD5 authentication if it's
-        specified. By default, the MD5 authenticaiton is disabled.
+        specified. By default, the MD5 authentication is disabled.
 
         ``multi_exit_disc`` specifies multi exit discriminator (MED) value.
         The default is None and if not specified, MED value is
@@ -284,17 +284,19 @@ class BGPSpeaker(object):
         ``is_next_hop_self`` specifies whether the BGP speaker announces
         its own ip address to iBGP neighbor or not as path's next_hop address.
 
-        ``connect_mode`` specifies how to connect to this neighbor.
-        CONNECT_MODE_ACTIVE tries to connect from us.
-        CONNECT_MODE_PASSIVE just listens and wait for the connection.
-        CONNECT_MODE_BOTH use both methods.
-        The default is CONNECT_MODE_BOTH
-
         ``local_address`` specifies Loopback interface address for
         iBGP peering.
 
         ``local_port`` specifies source TCP port for iBGP peering.
 
+        ``local_as`` specifies local AS number per-peer.
+        The default is the AS number of BGPSpeaker instance.
+
+        ``connect_mode`` specifies how to connect to this neighbor.
+        CONNECT_MODE_ACTIVE tries to connect from us.
+        CONNECT_MODE_PASSIVE just listens and wait for the connection.
+        CONNECT_MODE_BOTH use both methods.
+        The default is CONNECT_MODE_BOTH.
         """
         bgp_neighbor = {}
         bgp_neighbor[neighbors.IP_ADDRESS] = address
@@ -332,6 +334,9 @@ class BGPSpeaker(object):
         if local_port:
             bgp_neighbor[LOCAL_PORT] = local_port
 
+        if local_as:
+            bgp_neighbor[LOCAL_AS] = local_as
+
         call('neighbor.create', **bgp_neighbor)
 
     def neighbor_del(self, address):
